
#ifndef P1_META_DATA_H
#define P1_META_DATA_H

#include <memory>

#include <C_P1ImageCommonTypes.h>
#include <C_P1ImageCommonLinkage.h>
#include "C_P1ImageCommonMetaData.h"

#include "P1ImageBitmapImage.hpp"
#include "P1ImageRawImage.hpp"


namespace P1
{

namespace ImageSdk
{
    class EXPORT_API MetaObject : public P1::ImageSdkC::P1MetaObject
    {
    public:
        MetaObject() : P1::ImageSdkC::P1MetaObject(nullptr) {}

        MetaObject(P1::ImageSdkC::P1MetaObject& o) : P1::ImageSdkC::P1MetaObject(o.GetPtr()) {}

        /**
     * @brief Is meta object valid.
     *
     * @return A boolean flag that indicates if object is valid. True means it is valid
     */
        bool IsValid() { return this->P1::ImageSdkC::P1MetaObject::IsValid(); }

        /**
        * @brief Return the type of the object. <dni!> where to find the types.
        *
        * @return The type of the object is returned.
        */
        uint32_t GetType() const { return this->P1::ImageSdkC::P1MetaObject::GetType(); }

        /**
        * @brief Return the number of elements in the object.
        *
        * Use this function to get the number of elements in an array type metaobject. An object that contains a string will return the value 1.
        * @return Return the number of elements of the object.
        */
        uint32_t GetCount() const { return this->P1::ImageSdkC::P1MetaObject::GetCount(); }

        /**
        * @brief Get scalar unsigned byte from object. Can only be used on binary metaobjects with the correct type.
        *
        * It is the users responsability to use this function correctly. The type of the object can be retrieved by using the GetType() method.
        * If function is used on a wrong kind of object the function will set the error flag. A binary metaobject is eg. exif, gps and geotiff kind of metadata.
        * @param error: This parameter refers to a variable where function can store the error state.
        * @return The value of the object. If it was not successfull the error flag will be set and 0 is returned.
        */
        uint8_t GetUByte(bool& error) { return this->P1::ImageSdkC::P1MetaObject::GetUByte(error); }

        /**
        * @brief Get scalar unsigned word from object. Can only be used on binary metaobjects with the correct type.
        *
        * It is the users responsability to use this function correctly. The type of the object can be retrieved by using the GetType() method.
        * If function is used on a wrong kind of object the function will set the error flag. A binary metaobject is eg. exif, gps and geotiff kind of metadata.
        * @param error: This parameter refers to a variable where function can store the error state.
        * @return The value of the object. If it was not successfull the error flag will be set and 0 is returned.
        */
        uint16_t GetUWord(bool& error) { return this->P1::ImageSdkC::P1MetaObject::GetUWord(error); }

        /**
        * @brief Get scalar unsigned long from object. Can only be used on binary metaobjects with the correct type.
        *
        * It is the users responsability to use this function correctly. The type of the object can be retrieved by using the GetType() method.
        * If function is used on a wrong kind of object the function will set the error flag. A binary metaobject is eg. exif, gps and geotiff kind of metadata.
        * @param error: This parameter refers to a variable where function can store the error state.
        * @return The value of the object. If it was not successfull the error flag will be set and 0 is returned.
        */
        uint32_t GetULong(bool& error) { return this->P1::ImageSdkC::P1MetaObject::GetULong(error); }

        /**
        * @brief Get scalar signed byte from object. Can only be used on binary metaobjects with the correct type.
        *
        * It is the users responsability to use this function correctly. The type of the object can be retrieved by using the GetType() method.
        * If function is used on a wrong kind of object the function will set the error flag. A binary metaobject is eg. exif, gps and geotiff kind of metadata.
        * @param error: This parameter refers to a variable where function can store the error state.
        * @return The value of the object. If it was not successfull the error flag will be set and 0 is returned.
        */
        uint8_t GetSByte(bool& error) { return this->P1::ImageSdkC::P1MetaObject::GetSByte(error); }

        /**
        * @brief Get scalar signed word from object. Can only be used on binary metaobjects with the correct type.
        *
        * It is the users responsability to use this function correctly. The type of the object can be retrieved by using the GetType() method.
        * If function is used on a wrong kind of object the function will set the error flag. A binary metaobject is eg. exif, gps and geotiff kind of metadata.
        * @param error: This parameter refers to a variable where function can store the error state.
        * @return The value of the object. If it was not successfull the error flag will be set and 0 is returned.
        */
        uint16_t GetSWord(bool& error) { return this->P1::ImageSdkC::P1MetaObject::GetSWord(error); }

        /**
        * @brief Get scalar signed long from object. Can only be used on binary metaobjects with the correct type.
        *
        * It is the users responsability to use this function correctly. The type of the object can be retrieved by using the GetType() method.
        * If function is used on a wrong kind of object the function will set the error flag. A binary metaobject is eg. exif, gps and geotiff kind of metadata.
        * @param error: This parameter refers to a variable where function can store the error state.
        * @return The value of the object. If it was not successfull the error flag will be set and 0 is returned.
        */
        uint32_t GetSLong(bool& error) { return this->P1::ImageSdkC::P1MetaObject::GetSLong(error); }

        /**
        * @brief Get scalar float from object. Can only be used on binary metaobjects with the correct type.
        *
        * It is the users responsability to use this function correctly. The type of the object can be retrieved by using the GetType() method.
        * If function is used on a wrong kind of object the function will set the error flag. A binary metaobject is eg. exif, gps and geotiff kind of metadata.
        * @param error: This parameter refers to a variable where function can store the error state.
        * @return The value of the object. If it was not successfull the error flag will be set and 0 is returned.
        */
        float GetFloat(bool& error) { return this->P1::ImageSdkC::P1MetaObject::GetFloat(error); }

        /**
        * @brief Get scalar double from object. Can only be used on binary metaobjects with the correct type.
        *
        * It is the users responsability to use this function correctly. The type of the object can be retrieved by using the GetType() method.
        * If function is used on a wrong kind of object the function will set the error flag. A binary metaobject is eg. exif, gps and geotiff kind of metadata.
        * @param error: This parameter refers to a variable where function can store the error state.
        * @return The value of the object. If it was not successfull the error flag will be set and 0 is returned.
        */     
        double GetDouble(bool& error) { return this->P1::ImageSdkC::P1MetaObject::GetDouble(error); }

        /**
        * @brief Get scalar unsigned rational from object. Can only be used on binary metaobjects with the correct type.
        *
        * It is the users responsability to use this function correctly. The type of the object can be retrieved by using the GetType() method.
        * If function is used on a wrong kind of object the function will set the error flag. A binary metaobject is eg. exif, gps and geotiff kind of metadata.
        * A rational is value represented as a fraction (nom/denom) instead of a float or double. nom and denon is of type unsigned long
        * @param error: This parameter refers to a variable where function can store the error state.
        * @return The value of the object. If it was not successfull the error flag will be set and 0/0 is returned.
        */
        P1::ImageSdkC::CURational GetURational(bool& error) { return this->P1::ImageSdkC::P1MetaObject::GetURational(error); }

        /**
        * @brief Get scalar unsigned rational from object. Can only be used on binary metaobjects with the correct type.
        *
        * It is the users responsability to use this function correctly. The type of the object can be retrieved by using the GetType() method.
        * If function is used on a wrong kind of object the function will set the error flag. A binary metaobject is eg. exif, gps and geotiff kind of metadata.
        * A rational is value represented as a fraction (nom/denom) instead of a float or double. nom and denon is of type signed long
        * @param error: This parameter refers to a variable where function can store the error state.
        * @return The value of the object. If it was not successfull the error flag will be set and 0/0 is returned.
        */
        P1::ImageSdkC::CSRational GetSRational(bool& error) { return this->P1::ImageSdkC::P1MetaObject::GetSRational(error); }

        /**
        * @brief Get Raw string from object (including zero padding). Can be used on both binary and textual metaobjects. textual metaobject is e.g. an xmp value
        *
        * It is the users responsability to use this function correctly. The type of the object can be retrieved by using the GetType() method.
        * If function is used on a wrong kind of object the function will set the error flag.
        * @param error: This parameter refers to a variable where function can store the error state.
        * @return The value of the object. If it was not successfull the error flag will be set and "" is returned.
        */
        std::string GetRawString(bool& error) { return this->P1::ImageSdkC::P1MetaObject::GetRawString(error); }

        /**
        * @brief Get string from object (removes padding). Can be used on both binary and textual metaobjects. textual metaobject is e.g. an xmp value
        *
        * It is the users responsability to use this function correctly. The type of the object can be retrieved by using the GetType() method.
        * If function is used on a wrong kind of object the function will set the error flag.
        * @param error: This parameter refers to a variable where function can store the error state.
        * @return The value of the object. If it was not successfull the error flag will be set and "" is returned.
        */
        std::string GetString(bool& error) { return this->P1::ImageSdkC::P1MetaObject::GetString(error); }


        // Getters of arrays

        /** @brief Get array of unsigned bytes from object. Can only be used on binary metaobjects with the correct type.
        *
        * It is the users responsability to use this function correctly. The type of the object can be retrieved by using the GetType() method. The GetCount() method
        * returns the number of array elements, but is not needed, since function itself returns all elements
        * If function is used on a wrong kind of object the function will set the error flag. A binary metaobject is eg. exif, gps and geotiff kind of metadata.
        * @param error: This parameter refers to a variable where function can store the error state.
        * @return A vector with values of the object. If it was not successfull the error flag will be set and empty vector is returned.
        */
        std::vector<uint8_t> GetUBytes(bool& error) { return this->P1::ImageSdkC::P1MetaObject::GetUBytes(error); }

        /** @brief Get array of unsigned words from object. Can only be used on binary metaobjects with the correct type.
        *
        * It is the users responsability to use this function correctly. The type of the object can be retrieved by using the GetType() method. The GetCount() method
        * returns the number of array elements, but is not needed, since function itself returns all elements
        * If function is used on a wrong kind of object the function will set the error flag. A binary metaobject is eg. exif, gps and geotiff kind of metadata.
        * @param error: This parameter refers to a variable where function can store the error state.
        * @return A vector with values of the object. If it was not successfull the error flag will be set and empty vector is returned.
        */
        std::vector<uint16_t> GetUWords(bool& error) { return this->P1::ImageSdkC::P1MetaObject::GetUWords(error); }

        /** @brief Get array of unsigned longs from object. Can only be used on binary metaobjects with the correct type.
        *
        * It is the users responsability to use this function correctly. The type of the object can be retrieved by using the GetType() method. The GetCount() method
        * returns the number of array elements, but is not needed, since function itself returns all elements
        * If function is used on a wrong kind of object the function will set the error flag. A binary metaobject is eg. exif, gps and geotiff kind of metadata.
        * @param error: This parameter refers to a variable where function can store the error state.
        * @return A vector with values of the object. If it was not successfull the error flag will be set and empty vector is returned.
        */
        std::vector<uint32_t> GetULongs(bool& error) { return this->P1::ImageSdkC::P1MetaObject::GetULongs(error); }

        /** @brief Get array of signed bytes from object. Can only be used on binary metaobjects with the correct type.
        *
        * It is the users responsability to use this function correctly. The type of the object can be retrieved by using the GetType() method. The GetCount() method
        * returns the number of array elements, but is not needed, since function itself returns all elements
        * If function is used on a wrong kind of object the function will set the error flag. A binary metaobject is eg. exif, gps and geotiff kind of metadata.
        * @param error: This parameter refers to a variable where function can store the error state.
        * @return A vector with values of the object. If it was not successfull the error flag will be set and empty vector is returned.
        */
        std::vector<int8_t> GetSBytes(bool& error) { return this->P1::ImageSdkC::P1MetaObject::GetSBytes(error); }

        /** @brief Get array of signed words from object. Can only be used on binary metaobjects with the correct type.
        *
        * It is the users responsability to use this function correctly. The type of the object can be retrieved by using the GetType() method. The GetCount() method
        * returns the number of array elements, but is not needed, since function itself returns all elements
        * If function is used on a wrong kind of object the function will set the error flag. A binary metaobject is eg. exif, gps and geotiff kind of metadata.
        * @param error: This parameter refers to a variable where function can store the error state.
        * @return A vector with values of the object. If it was not successfull the error flag will be set and empty vector is returned.
        */
        std::vector<int16_t> GetSWords(bool& error) { return this->P1::ImageSdkC::P1MetaObject::GetSWords(error); }

        /** @brief Get array of signed longs from object. Can only be used on binary metaobjects with the correct type.
        *
        * It is the users responsability to use this function correctly. The type of the object can be retrieved by using the GetType() method. The GetCount() method
        * returns the number of array elements, but is not needed, since function itself returns all elements
        * If function is used on a wrong kind of object the function will set the error flag. A binary metaobject is eg. exif, gps and geotiff kind of metadata.
        * @param error: This parameter refers to a variable where function can store the error state.
        * @return A vector with values of the object. If it was not successfull the error flag will be set and empty vector is returned.
        */
        std::vector<int32_t> GetSLongs(bool& error) { return this->P1::ImageSdkC::P1MetaObject::GetSLongs(error); }

        /** @brief Get array of floats from object. Can only be used on binary metaobjects with the correct type.
        *
        * It is the users responsability to use this function correctly. The type of the object can be retrieved by using the GetType() method. The GetCount() method
        * returns the number of array elements, but is not needed, since function itself returns all elements
        * If function is used on a wrong kind of object the function will set the error flag. A binary metaobject is eg. exif, gps and geotiff kind of metadata.
        * @param error: This parameter refers to a variable where function can store the error state.
        * @return A vector with values of the object. If it was not successfull the error flag will be set and empty vector is returned.
        */
        std::vector<float> GetFloats(bool& error) { return this->P1::ImageSdkC::P1MetaObject::GetFloats(error); }

        /** @brief Get array of doubles from object. Can only be used on binary metaobjects with the correct type.
        *
        * It is the users responsability to use this function correctly. The type of the object can be retrieved by using the GetType() method. The GetCount() method
        * returns the number of array elements, but is not needed, since function itself returns all elements
        * If function is used on a wrong kind of object the function will set the error flag. A binary metaobject is eg. exif, gps and geotiff kind of metadata.
        * @param error: This parameter refers to a variable where function can store the error state.
        * @return A vector with values of the object. If it was not successfull the error flag will be set and empty vector is returned.
        */
        std::vector<double> GetDoubles(bool& error) { return this->P1::ImageSdkC::P1MetaObject::GetDoubles(error); }


        /**
        * @brief Set scalar unsigned byte value of object. Can only be used on binary metaobjects with the correct type.
        *
        * It is the users responsability to use this function correctly. The type of the object can be retrieved by using the GetType() method.
        * If function is used on a wrong kind of object the function will set the error flag. A binary metaobject is eg. exif, gps and geotiff kind of metadata.
        * @param value: This parameter is the value to be set
        * @return true is returned it setting the value was successfull, on error false is returned.
        */
        bool SetUByte(uint8_t value) { return this->P1::ImageSdkC::P1MetaObject::SetUByte(value); }

        /**
        * @brief Set scalar unsigned word value of object. Can only be used on binary metaobjects with the correct type.
        *
        * It is the users responsability to use this function correctly. The type of the object can be retrieved by using the GetType() method.
        * If function is used on a wrong kind of object the function will set the error flag. A binary metaobject is eg. exif, gps and geotiff kind of metadata.
        * @param value: This parameter is the value to be set
        * @return true is returned it setting the value was successfull, on error false is returned.
        */
        bool SetUWord(uint16_t value) { return this->P1::ImageSdkC::P1MetaObject::SetUWord(value); }

        /**
        * @brief Set scalar unsigned long value of object. Can only be used on binary metaobjects with the correct type.
        *
        * It is the users responsability to use this function correctly. The type of the object can be retrieved by using the GetType() method.
        * If function is used on a wrong kind of object the function will set the error flag. A binary metaobject is eg. exif, gps and geotiff kind of metadata.
        * @param value: This parameter is the value to be set
        * @return true is returned it setting the value was successfull, on error false is returned.
        */
        bool SetULong(uint32_t value) { return this->P1::ImageSdkC::P1MetaObject::SetULong(value); }

        /**
        * @brief Set scalar signed byte value of object. Can only be used on binary metaobjects with the correct type.
        *
        * It is the users responsability to use this function correctly. The type of the object can be retrieved by using the GetType() method.
        * If function is used on a wrong kind of object the function will set the error flag. A binary metaobject is eg. exif, gps and geotiff kind of metadata.
        * @param value: This parameter is the value to be set
        * @return true is returned it setting the value was successfull, on error false is returned.
        */
        bool SetSByte(uint8_t value) { return this->P1::ImageSdkC::P1MetaObject::SetSByte(value); }

        /**
        * @brief Set scalar signed word value of object. Can only be used on binary metaobjects with the correct type.
        *
        * It is the users responsability to use this function correctly. The type of the object can be retrieved by using the GetType() method.
        * If function is used on a wrong kind of object the function will set the error flag. A binary metaobject is eg. exif, gps and geotiff kind of metadata.
        * @param value: This parameter is the value to be set
        * @return true is returned it setting the value was successfull, on error false is returned.
        */
        bool SetSWord(uint16_t value) { return this->P1::ImageSdkC::P1MetaObject::SetSWord(value); }

        /**
        * @brief Set scalar signed long value of object. Can only be used on binary metaobjects with the correct type.
        *
        * It is the users responsability to use this function correctly. The type of the object can be retrieved by using the GetType() method.
        * If function is used on a wrong kind of object the function will set the error flag. A binary metaobject is eg. exif, gps and geotiff kind of metadata.
        * @param value: This parameter is the value to be set
        * @return true is returned it setting the value was successfull, on error false is returned.
        */
        bool SetSLong(uint32_t value) { return this->P1::ImageSdkC::P1MetaObject::SetSLong(value); }

        /**
        * @brief Set scalar float value of object. Can only be used on binary metaobjects with the correct type.
        *
        * It is the users responsability to use this function correctly. The type of the object can be retrieved by using the GetType() method.
        * If function is used on a wrong kind of object the function will set the error flag. A binary metaobject is eg. exif, gps and geotiff kind of metadata.
        * @param value: This parameter is the value to be set
        * @return true is returned it setting the value was successfull, on error false is returned.
        */
        bool SetFloat(float value) { return this->P1::ImageSdkC::P1MetaObject::SetFloat(value); }

        /**
        * @brief Set scalar double value of object. Can only be used on binary metaobjects with the correct type.
        *
        * It is the users responsability to use this function correctly. The type of the object can be retrieved by using the GetType() method.
        * If function is used on a wrong kind of object the function will set the error flag. A binary metaobject is eg. exif, gps and geotiff kind of metadata.
        * @param value: This parameter is the value to be set
        * @return true is returned it setting the value was successfull, on error false is returned.
        */
        bool SetDouble(double value) { return this->P1::ImageSdkC::P1MetaObject::SetDouble(value); }

        /**
        * @brief Set scalar unsigned rational value of object. Can only be used on binary metaobjects with the correct type.
        *
        * It is the users responsability to use this function correctly. The type of the object can be retrieved by using the GetType() method.
        * If function is used on a wrong kind of object the function will set the error flag. A binary metaobject is eg. exif, gps and geotiff kind of metadata.
        * A rational is value represented as a fraction (nom/denom) instead of a float or double. nom and denon is of type unsigned long
        * @param value: This parameter is the value to be set
        * @return true is returned it setting the value was successfull, on error false is returned.
        */
        bool SetURational(P1::ImageSdkC::CURational value) { return this->P1::ImageSdkC::P1MetaObject::SetURational(value); }

        /**
        * @brief Set scalar signed rational value of object. Can only be used on binary metaobjects with the correct type.
        *
        * It is the users responsability to use this function correctly. The type of the object can be retrieved by using the GetType() method.
        * If function is used on a wrong kind of object the function will set the error flag. A binary metaobject is eg. exif, gps and geotiff kind of metadata.
        * A rational is value represented as a fraction (nom/denom) instead of a float or double. nom and denon is of type signed long
        * @param value: This parameter is the value to be set
        * @return true is returned it setting the value was successfull, on error false is returned.
        */
        bool SetSRational(P1::ImageSdkC::CSRational value) { return this->P1::ImageSdkC::P1MetaObject::SetSRational(value); }

        /**
        * @brief Set string value of object. Can be used on both binary and textual metaobjects. textual metaobject is e.g. an xmp value
        *
        * It is the users responsability to use this function correctly. The type of the object can be retrieved by using the GetType() method.
        * If function is used on a wrong kind of object the function will set the error flag.
        * @param value: This parameter is the value to be set
        * @return true is returned it setting the value was successfull, on error false is returned.
        */
        bool SetString(std::string buffer) { return this->P1::ImageSdkC::P1MetaObject::SetString(buffer); }

        /** @brief Set array of unsigned bytes to object. Can only be used on binary metaobjects with the correct type.
        *
        * It is the users responsability to use this function correctly. The type of the object can be retrieved by using the GetType() method.
        * If function is used on a wrong kind of object the function will set the error flag. A binary metaobject is eg. exif, gps and geotiff kind of metadata.
        * @param values: Points to array of values to be set.
        * @param count: The number of values in the array to be set
        * @return true is returned it setting the value was successfull, on error false is returned.
        */
        bool SetUBytes(uint8_t* values, uint32_t count) { return this->P1::ImageSdkC::P1MetaObject::SetUBytes(values, count); }

        /** @brief Set array of unsigned words to object. Can only be used on binary metaobjects with the correct type.
        *
        * It is the users responsability to use this function correctly. The type of the object can be retrieved by using the GetType() method.
        * If function is used on a wrong kind of object the function will set the error flag. A binary metaobject is eg. exif, gps and geotiff kind of metadata.
        * @param values: Points to array of values to be set.
        * @param count: The number of values in the array to be set
        * @return true is returned it setting the value was successfull, on error false is returned.
        */
        bool SetUWords(uint16_t* values, uint32_t count) { return this->P1::ImageSdkC::P1MetaObject::SetUWords(values, count); }

        /** @brief Set array of unsigned longs to object. Can only be used on binary metaobjects with the correct type.
        *
        * It is the users responsability to use this function correctly. The type of the object can be retrieved by using the GetType() method.
        * If function is used on a wrong kind of object the function will set the error flag. A binary metaobject is eg. exif, gps and geotiff kind of metadata.
        * @param values: Points to array of values to be set.
        * @param count: The number of values in the array to be set
        * @return true is returned it setting the value was successfull, on error false is returned.
        */
        bool SetULongs(uint32_t* values, uint32_t count) { return this->P1::ImageSdkC::P1MetaObject::SetULongs(values, count); }

        /** @brief Set array of signed bytes to object. Can only be used on binary metaobjects with the correct type.
        *
        * It is the users responsability to use this function correctly. The type of the object can be retrieved by using the GetType() method.
        * If function is used on a wrong kind of object the function will set the error flag. A binary metaobject is eg. exif, gps and geotiff kind of metadata.
        * @param values: Points to array of values to be set.
        * @param count: The number of values in the array to be set
        * @return true is returned it setting the value was successfull, on error false is returned.
        */
        bool SetSBytes(int8_t* values, uint32_t count) { return this->P1::ImageSdkC::P1MetaObject::SetSBytes(values, count); }

        /** @brief Set array of signed words to object. Can only be used on binary metaobjects with the correct type.
        *
        * It is the users responsability to use this function correctly. The type of the object can be retrieved by using the GetType() method.
        * If function is used on a wrong kind of object the function will set the error flag. A binary metaobject is eg. exif, gps and geotiff kind of metadata.
        * @param values: Points to array of values to be set.
        * @param count: The number of values in the array to be set
        * @return true is returned it setting the value was successfull, on error false is returned.
        */
        bool SetSWords(int16_t* values, uint32_t count) { return this->P1::ImageSdkC::P1MetaObject::SetSWords(values, count); }

        /** @brief Set array of signed longs to object. Can only be used on binary metaobjects with the correct type.
        *
        * It is the users responsability to use this function correctly. The type of the object can be retrieved by using the GetType() method.
        * If function is used on a wrong kind of object the function will set the error flag. A binary metaobject is eg. exif, gps and geotiff kind of metadata.
        * @param values: Points to array of values to be set.
        * @param count: The number of values in the array to be set
        * @return true is returned it setting the value was successfull, on error false is returned.
        */
        bool SetSLongs(int32_t* values, uint32_t count) { return this->P1::ImageSdkC::P1MetaObject::SetSLongs(values, count); }

        /** @brief Set array of floats to object. Can only be used on binary metaobjects with the correct type.
        *
        * It is the users responsability to use this function correctly. The type of the object can be retrieved by using the GetType() method.
        * If function is used on a wrong kind of object the function will set the error flag. A binary metaobject is eg. exif, gps and geotiff kind of metadata.
        * @param values: Points to array of values to be set.
        * @param count: The number of values in the array to be set
        * @return true is returned it setting the value was successfull, on error false is returned.
        */
        bool SetFloats(float* values, uint32_t count) { return this->P1::ImageSdkC::P1MetaObject::SetFloats(values, count); }

        /** @brief Set array of doubles to object. Can only be used on binary metaobjects with the correct type.
        *
        * It is the users responsability to use this function correctly. The type of the object can be retrieved by using the GetType() method.
        * If function is used on a wrong kind of object the function will set the error flag. A binary metaobject is eg. exif, gps and geotiff kind of metadata.
        * @param values: Points to array of values to be set.
        * @param count: The number of values in the array to be set
        * @return true is returned it setting the value was successfull, on error false is returned.
        */
        bool SetDoubles(double* values, uint32_t count) { return this->P1::ImageSdkC::P1MetaObject::SetDoubles(values, count); }

        /** @brief Get state of readonly atttribute. If metaobject is readonly then it can only be read (using a getter function)
        *
        * Its the underlying system that determines if a value is readonly. The user cannot set this. The corresponding user functions is SetDisabled(bool state)
        * @return true is returned if it is readonly. otherwise false is returned.
        */
        bool IsReadOnly() { return this->P1::ImageSdkC::P1MetaObject::IsReadOnly(); }

        /** @brief Get state of writable atttribute. If metaobject writable then it will be written to output files, otherwise it will not.
        *
        * This is a way to control if a metaobject should be written to an output image file.
        * @return true is returned if it is writable. otherwise false is returned.
        */
        bool IsWritable() { return this->P1::ImageSdkC::P1MetaObject::IsWritable(); }

        /** @brief Set state of writable atttribute.
        *
        * With this method the user can decide if a metaobject is allowed to be written to an output image file.
        * @param state: Set the writable state of the metaobject. True means it is writable.
        */
        void SetWritable(bool state) { this->P1::ImageSdkC::P1MetaObject::SetWritable(state); }

        /** @brief Get state of disabled atttribute. If metaobject disabled then it cannot be changed.
        *
        * This is a user way of controlling access to the metaobject. it cannot overrule the readonly attribute.
        * @return true is returned if it is writable. otherwise false is returned.
        */
        bool IsDisabled() { return this->P1::ImageSdkC::P1MetaObject::IsDisabled(); }

        /** @brief Set state of disabled atttribute.
        *
        * With this method the user can protect the value in an object from being changed.
        * @param state: Set the disabled state of the metaobject. True means it is disabled.
        */
        void SetDisabled(bool state) { this->P1::ImageSdkC::P1MetaObject::SetDisabled(state); }

        /** @brief Is the metaobject modifiable. Use this function to check if its possible to modify the value of the metaobject
        *
        * It is just a convinience function to check if the readonly or disabled attribute has been set
        * @return true is returned if the value of the metaobject can be changed. otherwise false is returned.
        */
        bool IsModifiable() { return this->P1::ImageSdkC::P1MetaObject::IsModifiable(); }
    };
    
    class EXPORT_API MetaDataBase : public P1::ImageSdkC::P1MetaDataBase
    {
    public:
        MetaDataBase(RawImage& image, bool useDbFromFile = false);          

        /** @brief Retrieve a metadata object from the metadatabase
        *
        * A P1MetaData object will always be returned not matter if the metaobject was found or not.
        * To chech the validity of the object the IsValid() method can be used.
        * It is safe to nest both a GetMetaObject call with a getter or setter without checking for validity.
        * If metaobject is not found then an error will just be returned by the getter or setter.
        * @param path: The path to the object. The metaobjects are orginized in a tree structure where each level of the tree is divided by a ":"
        * @return a P1MetaObject. To check if an object was found can be checked with the IsValid() function. But it is safe to nest this function with a getter or setter.
        */
        MetaObject GetMetaObject(std::string path) 
        { 
            P1::ImageSdkC::P1MetaObject o = this->P1::ImageSdkC::P1MetaDataBase::GetMetaObject(path);
            MetaObject obj(o);
            return obj;            
        }

        /** @brief Create a new metadata object and insert it in the metadatabase
        *
        * A P1MetaData object will always be returned not matter if function was able to create object or not.
        * To chech the validity of the object the IsValid() method can be used.
        * It is safe to nest a CreateMetaObject call with a getter or setter without checking for validity. But the final error code or return value should be tested for success.
        * @param path: The path to the object that the user wants to create. The metaobjects are orginized in a tree structure where each level of the tree is divided by a ":". The last part
        * of the path must be a unique name of the new object.
        * @return a P1MetaObject. To check if an object was found can be checked with the IsValid() function. But it is safe to nest this function with a getter or setter.
        */
        MetaObject CreateMetaObject(std::string path)
        {
            P1::ImageSdkC::P1MetaObject o = this->P1::ImageSdkC::P1MetaDataBase::CreateMetaObject(path);
            MetaObject obj(o);
            return obj;
        }

        /** @brief Check for existance of a metaobject in the metadatabase
        *
        * @param path: The path to the object that the user wants to create. The metaobjects are orginized in a tree structure where each level of the tree is divided by a ":".
        * This function can also be used to check if a list of objects exist or not. Eg. a namespace in the xmp block.
        * @return A boolean value that indicates if metaobject exist or not. true means it does exist.
        */
        bool Exist(std::string path) { return this->P1::ImageSdkC::P1MetaDataBase::Exist(path); }

        /** @brief Create a new Xmp namespace in an xmp block. This requires that an xmp block exist in the database
        *
        * For now it is only possible to create new lists of objects in the xmp block. Therefore the function refers only to the xmp block
        * @param name: The name of the new namespace.
        * @param url: The url for the xmp namespace.
        * @return A boolean value that indicates if creation of the namespace was successfull or not.
        */
        bool CreateXmpNamespace(std::string name, std::string url) { return this->P1::ImageSdkC::P1MetaDataBase::CreateXmpNamespace(name, url); }

        /** @brief A debug function for internal use
        */
        void Dump() { return this->P1::ImageSdkC::P1MetaDataBase::Dump(); }

    };    
}
}

#endif
